// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This class dispays textual information from a plot. It's complicated because it considers whether everything can be shown in one box (when the x-axis is the same for every set of data), or when each set of data must be dispalyed separately (when one set of data has x-axis components different from another). 

import java.awt.*;

public class ShowSimData extends HideableFrame
{
	public ShowSimData (SimResults Results)
	{
		super("Text data for "+Results.Name);
		
		//Create menubar
		MenuBar menubar = new MenuBar();
		this.setMenuBar(menubar);

		//Create counter menu..
		Menu SD=new Menu("Close window");
		SD.add(new MenuItem("Close window"));
		menubar.add(SD);		
		
		// determine if everything can be in one table (all same x-points) or if we need multiple tables
		boolean anyDifference=false;
		if(Results.Lines.size()>0)
		{
			// see if any difference in number of x-points
			for(int i=0;i<Results.Lines.size();i++)
                        {
                            SimLine firstLine = (SimLine)Results.Lines.elementAt(i);
                            for(int j=i+1;j<Results.Lines.size();j++)
                            {
                                SimLine secondLine = (SimLine)Results.Lines.elementAt(j);
                                if(firstLine.Points.size() != secondLine.Points.size())
                                {
                                    anyDifference=true;
                                    i=Results.Lines.size();
                                    j=Results.Lines.size();
                                }
                            }
                        }

			if(!anyDifference)
			{
				// check if all x-points the same
                            for(int i=0;i<Results.Lines.size();i++)
                            {
                                SimLine firstLine = (SimLine)Results.Lines.elementAt(i);
                                for(int j=i+1;j<Results.Lines.size();j++)
                                {
                                    SimLine secondLine = (SimLine)Results.Lines.elementAt(j);

                                    for(int i2=0;i2<firstLine.Points.size();i2++)
                                    {
                                        ResultPair first = (ResultPair)firstLine.Points.elementAt(i2);
                                        ResultPair second = (ResultPair)secondLine.Points.elementAt(i2);
                                        
                                        if(first.x!= second.x)
                                        {
                                            anyDifference=true;
                                            
                                            i=Results.Lines.size();
                                            i2=firstLine.Points.size();
                                            j=Results.Lines.size();
                                        }
                                    }
                            }
                    }
                }
                
		if(anyDifference) // multiple text areas
		{
			this.setLayout(new GridLayout(1,Results.Lines.size()));
			TextArea [] t = new TextArea[Results.Lines.size()]; // separate text area for each data set

			for(int k=0;k<Results.Lines.size();k++)
			{
                                SimLine Line = (SimLine)Results.Lines.elementAt(k);
				int cellLength[] = new int[2];
				cellLength[0] = Results.Xaxis.length();
				cellLength[1] = Results.Yaxis.length();
				for(int i=0;i<Line.Points.size();i++)
				{
                                    ResultPair point = (ResultPair)Line.Points.elementAt(i);
                                    String xnum=""+point.x;
                                    String ynum=""+point.y;
                                    if(cellLength[0] < xnum.length() )
                                        cellLength[0] = xnum.length();
                                    if(cellLength[1] < ynum.length() )
                                        cellLength[1] = ynum.length();
				}

				StringBuffer data = new StringBuffer("\n[You can save this data on your personal computer by selecting,\n copying, and pasting into another program, e.g., spreadsheet \nor word processor. You will need to use the keyboard shortcut for \ncopying. Usually this is <control>-c or <command>-c.]\n\n");

				data.append("Results for "+Line.Name+"\n\n");

				data.append(Results.Xaxis);
				for(int i=0;i<cellLength[0]-Results.Xaxis.length();i++)
					data.append(" ");
				data.append("\t"+Results.Yaxis);
				for(int i=0;i<cellLength[1]-Results.Xaxis.length();i++)
					data.append(" ");

				data.append("\n");

				for(int j=0;j<Line.Points.size();j++)
				{
                                    ResultPair point = (ResultPair)Line.Points.elementAt(j);
                                    String xnum=""+point.x;
                                    data.append(xnum);
                                    for(int i=0;i<cellLength[0]-xnum.length();i++)
                                        data.append(" ");
                                    data.append("\t");
                                    String ynum=""+point.y;
                                    data.append(ynum);
                                    for(int i=0;i<cellLength[1]-ynum.length();i++)
                                        data.append(" ");
                                    data.append("\n");
				}

				t[k] = new TextArea(data.toString(), Line.Points.size(), 25);
				t[k].setFont(new Font ("Courier", 1, 12) );
				t[k].setEditable(false);

				this.add(t[k]);
			}
		}
		else   // one text area for everything
		{
			// find maximum size of each column
			int cellLength[] = new int[Results.Lines.size()+1];
                        SimLine firstLine = (SimLine)Results.Lines.elementAt(0);

			cellLength[0] = Results.Xaxis.length();
			// find size of first column (x-data)
                        for(int i=0;i<firstLine.Points.size();i++)
			{
                            ResultPair point = (ResultPair)firstLine.Points.elementAt(i);
                            String xnum=""+point.x;
                            if(cellLength[0] < xnum.length() )
                                cellLength[0] = xnum.length();
			}
			// find size of each of the other columns (y-data)
			for(int k=0;k<Results.Lines.size();k++)
			{
                            SimLine Line = (SimLine)Results.Lines.elementAt(k);
                            cellLength[k+1] = Line.Name.length();

                            // go through data
                            for(int i=0;i<Line.Points.size();i++)
                            {
                                ResultPair point = (ResultPair)firstLine.Points.elementAt(i);
                                String ynum=""+point.y;
                                if(cellLength[k+1] < ynum.length() )
                                    cellLength[k+1] = ynum.length();
                            }
			}
			StringBuffer data = new StringBuffer("\n[You can save this data on your personal computer by selecting,\n copying, and pasting into another program, e.g., spreadsheet \nor word processor. You will need to use the keyboard shortcut for \ncopying. Usually this is <control>-c or <alt>-c.]\n\n");
                        
                        data.append("\t"+Results.Yaxis+"\n\n");

			// first row (headers)
			data.append(Results.Xaxis);
			for(int i=0;i<cellLength[0]-Results.Xaxis.length();i++)
				data.append(" ");
			for(int k=0;k<Results.Lines.size();k++)
			{
                            SimLine Line = (SimLine)Results.Lines.elementAt(k);
                            data.append("\t"+Line.Name );
                            for(int i=0;i<cellLength[k+1]-Line.Name.length();i++)
                                data.append(" ");
			}
			data.append("\n");

			// each subsequent row
                        for(int j=0;j<firstLine.Points.size(); j++)
                        {
                            // x value
                            ResultPair point = (ResultPair)firstLine.Points.elementAt(j);
                            String xnum=""+point.x;
                            data.append(xnum);
                            for(int i=0;i<cellLength[0]-xnum.length();i++)
                                data.append(" ");
                                      
                            // each y value
                            for(int k=0;k<Results.Lines.size();k++)
                            {
                                SimLine Line = (SimLine)Results.Lines.elementAt(k);
                                data.append("\t");
                               
                                ResultPair point2 = (ResultPair)Line.Points.elementAt(j);
                                String ynum=""+point2.y;
                                data.append(ynum);
                                for(int i=0;i<cellLength[1]-ynum.length();i++)
                                    data.append(" ");
                            }
                            data.append("\n");
                        }
			this.setLayout(new GridLayout(1,1));
			TextArea t = new TextArea(data.toString(), Results.Lines.size(), 25);
			t.setFont(new Font ("Courier", 1, 12) );
			t.setEditable(false);

			this.add(t);
		}
            }

		this.resize(500,500);
	}
	
	public boolean handleEvent (Event e)
	{
		switch(e.id)
		{
			case Event.ACTION_EVENT:
			if(e.target instanceof MenuItem)
			{
				if (((String)e.arg).equals("Close window"))
					this.hide();
				return true;
			}
			case Event.WINDOW_DESTROY:
				hide();
				return true;
			default:
				return false;
		}
	}	
}