// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This abstract class defines methods used for all Simulations
import java.io.*;
import java.util.Vector;

public abstract class Simulation implements Serializable, Runnable
{
	// The following is defined specifically for a given simulation, it allows us to fill in the necessary parameters, the various methods will know how to treat them and the GUI will know how to organize them to work properly
	Parameter [] IVParameters, simParameters, outputParameters, Variables;
	String Name;
        String Description;
        String Status;
        Parameter [] IVParametersDefault, simParametersDefault, outputParametersDefault, variablesDefault;
        SimResults Results;
        boolean done;
        Thread animate;
        String CurrentRunName;

        public abstract boolean UpdateParameters(Parameter[] newModelParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters);

	public abstract boolean CheckParameters();

	public abstract void RunSimulation(String conditionName);
        
        public abstract void startRun(String conditionName);
}
