// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This program provides the graphical user interface (GUI) for the various simulations.

import java.awt.*;
import java.util.*;
import java.util.Date;
import java.awt.event.*;
                                                                
public class SimulationGUI extends CloseableFrame
{
        Simulation sim;
        TextField[] IVParametersTF;
        Button[] IVParametersB;
        TextField [] simParametersTF;
        Button[] simParametersB;
        TextField [] outputParametersTF;
        Button[] outputParametersB;
        Parameter [] outputParameters;
        Checkbox clearPlotCB;
        TextField simNameTF;
        Choice IVchoice;
        String conditionName="";
        Button startSimulationB;
        	
	public SimulationGUI(Simulation sim)
	{
            super(sim.Name);
            this.sim = sim;

            //Create menubar
            MenuBar menubar = new MenuBar();
            this.setMenuBar(menubar);
            //Create counter menu..
            Menu SD=new Menu("Model description");
            MenuItem SDI = new MenuItem("Show description");
            final String simName = sim.Name;
            final String simDescrp = sim.Description;
             SDI.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent event)
				{
                                    ParameterWindow temp = new ParameterWindow(simName, simDescrp);
                                }});
            SD.add(SDI);
            menubar.add(SD);
            
            // set up window to display parameters
            this.setLayout(new BorderLayout(10,10) );
            
            // Independet Variables parameters on left panel
            Panel leftPanel = new Panel();
            leftPanel.setLayout(new BorderLayout(10,10) );
            
            Panel topIVPanel = new Panel();
            topIVPanel.setLayout(new GridLayout(2,1) );
            topIVPanel.add(new Label ("Choose the independent variable") );
            IVchoice = new Choice();
            for(int i=0;i<sim.simParameters.length;i++)
                IVchoice.add(sim.simParameters[i].Name);
            IVchoice.select((int)sim.IVParameters[0].Value);
            topIVPanel.add(IVchoice);
            leftPanel.add("North", topIVPanel);       
            Panel nameIVParametersPanel = new Panel();
            Panel valIVParametersPanel = new Panel();
            Panel descIVParametersPanel = new Panel();
            nameIVParametersPanel.setLayout( new GridLayout(sim.IVParameters.length+1,1) );   
            valIVParametersPanel.setLayout( new GridLayout(sim.IVParameters.length+1,1) );     
            descIVParametersPanel.setLayout( new GridLayout(sim.IVParameters.length+1,1) );                           
            // headers
            nameIVParametersPanel.add(new Label("Name") );
            valIVParametersPanel.add(new Label("Value") ); 
            descIVParametersPanel.add(new Label("Description") );  
            
            //Parameters
            IVParametersTF = new TextField[sim.IVParameters.length];
            IVParametersB = new Button[sim.IVParameters.length];
            for(int i=1;i<sim.IVParameters.length;i++)
            {
                nameIVParametersPanel.add(new Label(sim.IVParameters[i].Name) );
                IVParametersTF[i] = new TextField(""+sim.IVParameters[i].Value);
                valIVParametersPanel.add(IVParametersTF[i]);
                IVParametersB[i] = new Button("Description");
                final Parameter tempParam = sim.IVParameters[i];
                IVParametersB[i].addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent event)
				{
                                    ParameterWindow temp = new ParameterWindow(tempParam.Name, tempParam.Description);
                                }});
                descIVParametersPanel.add(IVParametersB[i]);                 
            }
            leftPanel.add("West", nameIVParametersPanel);
            leftPanel.add("Center", valIVParametersPanel);
            leftPanel.add("East", descIVParametersPanel);
            
            // Simulation parameters on center panel
            Panel CenterPanel = new Panel();
            CenterPanel.setLayout(new BorderLayout(3,3) );
            CenterPanel.add("North", new Label ("Model parameters", Label.CENTER) );
            Panel namesimParametersPanel = new Panel();
            namesimParametersPanel.setLayout( new GridLayout(sim.simParameters.length+1,1,5,5) );
            Panel valsimParametersPanel = new Panel();
            valsimParametersPanel.setLayout( new GridLayout(sim.simParameters.length+1,1,5,5) );
            Panel descsimParametersPanel = new Panel();
            descsimParametersPanel.setLayout( new GridLayout(sim.simParameters.length+1,1,5,5) );
            
            // headers
            namesimParametersPanel.add(new Label("Name") );
            valsimParametersPanel.add(new Label("Value") ); 
            descsimParametersPanel.add(new Label("Description") );  
            
            //Parameters
            simParametersTF = new TextField[sim.simParameters.length];
            simParametersB = new Button[sim.simParameters.length];
            for(int i=0;i<sim.simParameters.length;i++)
            {
                namesimParametersPanel.add(new Label(sim.simParameters[i].Name) );
                simParametersTF[i] = new TextField(""+sim.simParameters[i].Value);
                valsimParametersPanel.add(simParametersTF[i]);
                simParametersB[i] = new Button("Description");
                final Parameter tempParam = sim.simParameters[i];
                simParametersB[i].addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent event)
				{
                                    ParameterWindow temp = new ParameterWindow(tempParam.Name, tempParam.Description);
                                }});
                descsimParametersPanel.add(simParametersB[i]);                 
            }
            CenterPanel.add("West", namesimParametersPanel);                                
            CenterPanel.add("Center", valsimParametersPanel);
            CenterPanel.add("East", descsimParametersPanel);
  
             // Put the panels on the frame
            this.add("West", leftPanel);
            ScrollPane CenterSP = new ScrollPane();
            CenterSP.setSize(250,100);
            CenterSP.add(CenterPanel);
            this.add("Center", CenterSP);
            
            Panel bottomPanel = new Panel();
            bottomPanel.setLayout(new BorderLayout() );
                        
            Panel simPanel = new Panel();
            simPanel.setLayout(new BorderLayout());
            
            startSimulationB = new Button("Start simulation");
            startSimulationB.addActionListener(new ActionListener(){
				public void actionPerformed (ActionEvent event)
				{
                                    StartTheSimulation();
                                }});
            simPanel.add("East", startSimulationB);
            clearPlotCB = new Checkbox("Clear plot", null, true);
            simPanel.add("West", clearPlotCB);
            simNameTF = new TextField("Enter label here");
            Panel simNameP = new Panel();
            simNameP.setLayout(new GridLayout(2,1) );
            simNameP.add(new Label("Simulation name") );
            simNameP.add(simNameTF);
            simPanel.add("Center", simNameP);
            bottomPanel.add("North", simPanel);
            this.add("South", bottomPanel);
            this.pack();
            this.show();
	}
        
        public void StartTheSimulation()
        {
            startSimulationB.setLabel("Running");
            startSimulationB.setEnabled(false);
            
            // get IV parameters from window
            Parameter[] newIVParameters = new Parameter[sim.IVParameters.length];
            int newIVchoiceIndex = IVchoice.getSelectedIndex(), oldIVchoiceIndex = (int)sim.IVParameters[0].Value;
            for(int i=0;i<newIVParameters.length;i++)
            {
                newIVParameters[i] = sim.IVParameters[i].copy();
                if(i>0)
                    newIVParameters[i].Value = Double.valueOf(IVParametersTF[i].getText()).doubleValue();
            }
            newIVParameters[0].Value = newIVchoiceIndex;
            
            // get Sim parameters from window
            Parameter[] newsimParameters = new Parameter[sim.simParameters.length];
            for(int i=0;i<newsimParameters.length;i++)
            {
                newsimParameters[i] = sim.simParameters[i].copy();
                newsimParameters[i].Value = Double.valueOf(simParametersTF[i].getText()).doubleValue();
            }           
            
            // get Output parameters from window
            // if requested, clear the plot
            if(clearPlotCB.getState())
                sim.Results.clear();
            
            conditionName = simNameTF.getText();
            
            sim.UpdateParameters(newIVParameters, newsimParameters,null);
            
            // Redefine the plot if the IV has changed, clears it too
            if(newIVchoiceIndex != oldIVchoiceIndex)
                sim.Results = new SimResults(sim.Name, sim.simParameters[(int)sim.IVParameters[0].Value].Name, "Percept strength");           
   
            sim.Status=""+0;
            sim.startRun(conditionName);
            // loop through until simulation is done
             Date now = new Date();
            long markTime = (long)now.getTime();           
            
            while(!sim.done)
            {
                startSimulationB.setLabel(sim.Status+"% done");
                markTime = (long)now.getTime();
                // wait half a second before checking again
                while ((long)now.getTime() - markTime<500)
                    now = new Date();
            }

            sim.Results.ShowResults(); 
            startSimulationB.setLabel("Start simulation");
            startSimulationB.setEnabled(true);
        }
        
   
}
				  